#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <iomanip.h>
#include "JSSProblem.h"
#include "JSSPReader.h"
#include "JSSPSolution.h"
#include "JSSPMutation.h"
#include "JSSPInitializer.h"
#include "JSSPCrossover.h"
#include "JSSPFitnessFunction.h"
#include "../deffunc.h"
#include <new.h>
#include <Sort_Recombinator.h>
#include <SimilarParentSelector.h>
#include <GeneticAlgorithm.h>

typedef long time_t;
typedef long clock_t;

void allocation_handler()
{
   cout << "Out of Memory. Aborting Program...."<< endl;
   exit(1);
}

void solvewithGASTD(JSSProblem JSSPInstance)
{
   JSSPFitnessFunction   fitFunc;
   JSSPInitializer       Initme(&JSSPInstance);
   JSSPMutation          Jmutation;
   JSSPCrossover         Jcross;

   Sort_Recombinator     SRecombinator;
   SimilarParentSelector SimilarSelect;
    
   GeneticAlgorithm mySolver(&fitFunc,&Jcross,&Jmutation,&SimilarSelect,&SRecombinator,&Initme);

   int MaxGenNum= 100;
   int PopulationSize = 10;
   float MutationProbability = 0.2;
   float CrossoverProbability = 0.6;
   int TestTimes = 10;
   int TestType = 2;

   mySolver.SetMaxGeneration(MaxGenNum);
   mySolver.SetPopulationSize(PopulationSize);
   mySolver.SetMutationProb(MutationProbability);
   mySolver.SetCrossProb(CrossoverProbability);
   //mySolver.run();
   if(TestType>0&&TestType<4) mySolver.setTestType(TestType);

   cout << " before runFuzzyTimes() " << endl;
   mySolver.runFuzzyTimes(maximum(1,TestTimes));
   cout << " after runFuzzyTimes() " << endl;

   return;

}// end of solvewithGASTD()

void solvewithGA(JSSProblem JSSPInstance)
{
   JSSPFitnessFunction   fitFunc;
   JSSPInitializer       Initme(&JSSPInstance);
   JSSPMutation          Jmutation;
   JSSPCrossover         Jcross;

   Sort_Recombinator     SRecombinator;
   SimilarParentSelector SimilarSelect;
    
   cout << " I defined everything " << endl;

   GeneticAlgorithm mySolver(&fitFunc,&Jcross,&Jmutation,&SimilarSelect,&SRecombinator,&Initme);

   cout << " I defined GA also " << endl;
   
   
   int START = 0, noRun=1;
   int MaxGenNum;
   int PopulationSize;
   float MutationProbability;
   float CrossoverProbability;
   int TestTimes;
   int TestType;
   char Answer[4];


   while(!START)
   {  
      cout << " Enter the Generation Number : ";
      cin  >> setw(4) >> MaxGenNum;

      cout << " Enter the Population Size : ";
      cin  >> setw(4) >> PopulationSize;

      cout << " Enter the Mutation Probability : ";
      cin  >> setw(4) >> MutationProbability;

      cout << " Enter the Crossover Probability : ";
      cin  >> setw(9) >> CrossoverProbability;

      cout << " Enter the number of Try : ";
      cin  >> setw(4) >> TestTimes;

      cout << " Enter the Test Type (0:standart, 1:Cross, 2:Mutation, 3: Cross&Mutation : ";
      cin  >> setw(2) >> TestType;


      cout << " Do you want to change anything ?: ";
      cin  >> setw(4) >> Answer;
      if(!strcmp(Answer,"N")||!strcmp(Answer,"No")|| 
	 !strcmp(Answer,"n")||!strcmp(Answer,"no"))
      {  START = 1;
         cout << " Do you want to run the GA ?: ";
         cin  >> setw(4) >> Answer;
         if(!strcmp(Answer,"N")||!strcmp(Answer,"No")||
	    !strcmp(Answer,"n")||!strcmp(Answer,"no")) noRun = 0;
      }
   }// end of While

   time_t start;
   time_t stop;
   cout << "\n with GA Time: " << ctime(&start) << endl;

   if(noRun)
   {   
       mySolver.SetMaxGeneration(MaxGenNum);
       mySolver.SetPopulationSize(PopulationSize);
       mySolver.SetMutationProb(MutationProbability);
       mySolver.SetCrossProb(CrossoverProbability);
       if(TestType>0&&TestType<4) mySolver.setTestType(TestType);
       cout << " before run" << endl;
       mySolver.runFuzzyTimes(maximum(1,TestTimes));
       //mySolver.run();

   }// end of if

   cout << "the end Time: " << ctime(&stop) << endl;
   cout << " The process took " << difftime(stop,start) << " secs " << endl;
   cout << " The process took " << start << "-"<< stop << " secs " << endl;

   return;
}


main(int argc, char *argv[])
{  JSSPReader  JSSReader;
   int i;


   srand(150);
   // Set Allocation Exception Handler

   set_new_handler(allocation_handler);

   printf("\n 1st oparam %s ",argv[0]);
   printf("\n 1nd oparam %s ",argv[1]);
   
   if(argc>1) JSSReader.setFileName(argv[1]); 

   JSSProblem JSSPInstance;
   JSSReader.read(JSSPInstance);
   JSSPInstance.print();

   /*
   JSSPSolution sol(JSSPInstance);
   sol.solve();
   //sol.print();
   cout << sol;
   JSSPSolution sol2(JSSPInstance);
   sol2.solveRandom();
   cout << sol2;
   sol2.restore();
   cout << sol2;

   JSSPSolution *Probs =new JSSPSolution[10];

   for(i=0;i<10;i++) Probs[i].setProblem(JSSPInstance);

   for(i=0;i<10;i++) Probs[i].solveRandom();

   for(i=0;i<10;i++) Probs[i].restore();

   int min=0, minval=Probs[min].getMakespan(), next;

   int tot=0;
   for(i=0;i<10;i++)
   {  next = Probs[i].getMakespan();
      tot +=next;
      if(minval>next) { min=i; minval=next;}
   }// end of for

   for(i=0;i<10;i++) cout << "Problem : " << i << endl << Probs[i];

   cout << "The best solution is "<< endl << Probs[min] << endl;


   cout << "PROB#  Makespan" << endl;
   for(i=0;i<10;i++)
       cout << setw(5) << i << setw(10) << Probs[i].getMakespan() << endl;
   cout << endl << "The Population average is " << tot*0.1 << endl;
   cout << "after the print " << endl;

   */

   cout << "\n with GA " << endl;
   //solvewithGA(JSSPInstance);
   solvewithGASTD(JSSPInstance);
   cout << "\n end GA " << endl;

}// end of main()

